//-------------------------------------------------------
// Coefficients of One-dimensional N-Atom Network (CONAN)
//-------------------------------------------------------

//    Copyright (C) 2016  The Few-Body Group, Department for Physics
//    and Astronomy, Aarhus University, Denmark

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.


//
// Created by jesper on 1/25/16.
//

#ifndef CONAN_CVECTOR_H
#define CONAN_CVECTOR_H


#include <mpfr.h>
#include <gsl/gsl_matrix_double.h>

//-------------------------------
// Basic Vector Library
//-------------------------------

extern int prec;
typedef struct {
    int dim;
    mpfr_t* data;
} vector;

void vectorinit(vector* v, int dimension);

void vectorset(vector v, int entry, mpfr_t value);

void vectorset_d(vector v, int entry, double value);

void vectorget(vector v, int entry, mpfr_t output);

void vectorfree(vector* v);

void vectorscale(vector v, mpfr_t scale, vector result);

void vectorscale_d(vector v, double scale, vector result);

// Stores v-w in result.
void vectorsub(vector v, vector w, vector result);

void vectoradd(vector v, vector w, vector result);

// Shifts every entry in the vector by +1.
void vectorshift(vector v, vector result);

void vectorset_all_d(vector v, double d);

void vectordiv(vector v, vector w, vector result); // v/w entrywise;

//-------------------------------
// Basic Matrix Library
//-------------------------------

typedef struct {
    int row;
    int col;
    mpfr_t* data;
} matrix;

void matrixinit(matrix* M, int rows, int columns);

void matrixset(matrix M, int row, int column, mpfr_t value);

void matrixset_d(matrix M, int row, int column, double value);

void matrixget(matrix M, int row, int column, mpfr_t output);

void matrixfree(matrix* M);

void matrixset_I(matrix M);

void matrixset_Mat_Sym(gsl_matrix* A, matrix M);

//-------------------------------
// Mixed Matrix/Vector functions
//-------------------------------

// Multiplies a vector by Row i, and takes the dotproduct with Row j.
double RowDotVecMulRow(vector v, matrix M, int j, int i);

#endif //CONAN_CVECTOR_H
