//-------------------------------------------------------
// Coefficients of One-dimensional N-Atom Network (CONAN)
//-------------------------------------------------------

//    Copyright (C) 2016  The Few-Body Group, Department for Physics
//    and Astronomy, Aarhus University, Denmark

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.


//
// Created by jesper on 1/31/16.
//

#include "output.h"
#include "expparser.h"

char* delim = " ";

void output_header(FILE* stream, const char* potential, char** variables, int n) {

	fprintf(stream, "N%s%i\n", delim, N);
	fprintf(stream, "V%s%s\n", delim, potential);
	fprintf(stream, "L%s%g\n", delim, wellwidth);
	fprintf(stream, "b%s%i\n", delim, basesize);
	fprintf(stream, "p%s%i\n", delim, prec);
	fprintf(stream, "abs-solver%s%g\n", delim, int_prec_abs_Solver);
	fprintf(stream, "rel-solver%s%g\n", delim, int_prec_rel_Solver);
	fprintf(stream, "abs-final%s%g\n", delim, int_prec_abs_final);
	fprintf(stream, "rel-final%s%g\n", delim, int_prec_rel_final);

	fprintf(stream, "Parameters");
	for(int i=0; i<n; i++) {
		fprintf(stream, "%s%s", delim, variables[i]);
	}
	fprintf(stream, "\n");
}

void output_default(FILE* stream, struct result* r, double* vals, int n) {
		fprintf(stream, "P");
		for(int k=0; k<n; k++) {
			fprintf(stream, "%s%g",delim, vals[k]);
		}
		fprintf(stream, "\n");

		fprintf(stream, "I");
		for(int k=0; k<N-1; k++) {
			fprintf(stream, "%s%g", delim, gsl_vector_get(r->Ik, k));
		}
		fprintf(stream, "\n");

		fprintf(stream, "E");
		for(int k=0; k<N; k++) {
			fprintf(stream, "%s%g", delim, gsl_vector_get(r->E, k));
		}
		fprintf(stream, "\n");
}

void output_potential(char** names, double* vals, int n,  int i) {
	char filename[256];
	sprintf(filename, "potential_%i.dat", i);
	FILE* foutput = fopen(filename, "w");

	for(int k=0; k<n; k++) {
		fprintf(foutput, "%s=%g  ",names[k], vals[k]);
	}
	fprintf(foutput, "\n");

	double dx = wellwidth / 1000.0;
	for(double x=0; x<wellwidth; x+=dx) {
		double y = eval(potential, x, wellwidth);
		fprintf(foutput, "%g, %g\n", x, y);
	}
	fclose(foutput);
}
