//-------------------------------------------------------
// Coefficients of One-dimensional N-Atom Network (CONAN)
//-------------------------------------------------------

//    Copyright (C) 2016  The Few-Body Group, Department for Physics
//    and Astronomy, Aarhus University, Denmark

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.


//
// Created by jesper on 1/31/16.
//

#ifndef CONAN_PHYSICS_H
#define CONAN_PHYSICS_H

//-------------------------------
// Dmitri's Jacobi-Diagonalization-Algorithm
//-------------------------------

#include <gsl/gsl_matrix_double.h>
#include <gsl/gsl_sf_gamma.h>
#include "cvector.h"

// Size of expansion well.
extern double wellwidth;

// Number of atoms.
extern int N;

// Size of expansion states
extern int basesize;

// Precision for arbitrary precision library.
//static int prec = 256;

// Potential function.
extern struct expression* potential;



// Precision for Quantum-solver integrations.
extern double int_prec_abs_Solver;
extern double int_prec_rel_Solver;

// Precision of the final integrations
extern double int_prec_abs_final;
extern double int_prec_rel_final;

struct result {
    gsl_vector* Ik;
    gsl_vector* E;
};

typedef struct {
    int dim;
    int k;
    gsl_vector* E;
    gsl_matrix* C;
} intparams;

typedef struct {
    int* m;
    int* k;
} intparams_V;

double V(double x);

void phi_func(mpfr_t app, mpfr_t aqq, mpfr_t apq, mpfr_t result);

void app1_func(mpfr_t c, mpfr_t s, mpfr_t app, mpfr_t apq, mpfr_t aqq,
               mpfr_t result);

void aqq1_func(mpfr_t c, mpfr_t s, mpfr_t app, mpfr_t apq, mpfr_t aqq,
               mpfr_t result);

void cx_minus_sy(mpfr_t x, mpfr_t y, mpfr_t c, mpfr_t s, mpfr_t result);

void cx_plus_sy(mpfr_t x, mpfr_t y, mpfr_t c, mpfr_t s, mpfr_t result);

// e = eigenvalues, V = eigenvectors, upper triangle of A is destroyed.
int Jacobi(matrix A, vector e, matrix U);

//-------------------------------
// Solver for the quantum system
//-------------------------------

double integrand_V(double t, void* params_ptr);

void Hamiltonian(gsl_matrix* H);

void Solver(gsl_matrix* C, gsl_vector* E);

double Psi(double x, int i, gsl_matrix* C);

double DPsi(double x, int i, gsl_matrix* C);

//-------------------------------
// Generating the B(x)-matrix
//-------------------------------

void B_Gen(double x, gsl_matrix* C, matrix B);

//-------------------------------
// Functions for calculating the derivatives
//-------------------------------

void Polynomial_Coefficients(vector eigenval, vector result);

void Deriv_Matrix(vector eigenval, vector p, int k, vector result);

//-------------------------------
// Constructing the integrand
//-------------------------------

double Integrand(double x, void* params_ptr);

//-------------------------------
// Final Coefficients
//-------------------------------

double I(int dim, int k, gsl_vector* E, gsl_matrix* C);

void free_result(struct result* r);

#endif //CONAN_PHYSICS_H
